// Daniel Shiffman
// Tracking the average location beyond a given depth threshold
// Thanks to Dan O'Sullivan
// http://www.shiffman.net
// https://github.com/shiffman/libfreenect/tree/master/wrappers/java/processing

import org.openkinect.*;
import org.openkinect.processing.*;
import processing.net.*;


// Showing how we can farm all the kinect stuff out to a separate class
KinectTracker tracker;
// Kinect Library object
Kinect kinect;

//
Server server ;
Client client ;

boolean connected ;

String rcvString;


void setup() 
{
  size(640,520);
  frameRate(60);
  kinect = new Kinect(this);
  tracker = new KinectTracker();
  //
  server = new Server(this, 5204) ;
}


void draw() 
{
  background(255);

  // Run the tracking analysis
  tracker.track();
  // Show the image
  tracker.display();

  // Let's draw the raw location
  PVector v1 = tracker.getPos();
  fill(50,100,250,200);
  noStroke();
  ellipse(v1.x,v1.y,20,20);

  // Let's draw the "lerped" location
  PVector v2 = tracker.getLerpedPos();
  fill(100,250,50,200);
  noStroke();
  ellipse(v2.x,v2.y,20,20);

  // Display some info
  int t = tracker.getThreshold();
  fill(0);
  text("threshold: " + t + "    " +  "framerate: " + (int)frameRate + "    " + "UP increase threshold, DOWN decrease threshold" + "\nReceiveString: " + rcvString, 8, 500);

  //
  updateSocket(v1.x, v1.y);
  //updateSocket(v1.x, v1.y, tracker.display);
  
  receivedSocket();
}


void keyPressed() 
{
  int t = tracker.getThreshold();
  if (key == CODED) 
  {
    if (keyCode == UP) 
    {
      t+=5;
      tracker.setThreshold(t);
    } 
    else if (keyCode == DOWN)
    {
      t-=5;
      tracker.setThreshold(t);
    }
  }
}


void stop() 
{
  tracker.quit();
  super.stop();
}



// ----- Socket Sender ------ 
void updateSocket(float px, float py) 
{
  String str = px + "," + py;
  server.write(str) ;
  //print(str);
}


// ----- Socket Receiver ------
void receivedSocket()
{
  if (client != null) 
  {
    String data = client.readString();
    
    if (data != null) 
    {
      //println(data);
      rcvString = data;
    }
  }
}

void serverEvent(Server srv, Client clt) 
{
  println("connected") ;
  if (client != null) server.disconnect(client) ;
  client = clt ;
}

